from __future__ import print_function
import begin

@begin.subcommand
def name(answer):
    "What is your name?"
    print(answer)

@begin.subcommand
def quest(answer):
    "What is your quest?"
    print(answer)

@begin.subcommand
def colour(answer):
    "What is your favourite colour?"
    print(answer)

@begin.start(cmd_delim='--')
def main():
    pass



import begin

@begin.subcommand
def name(answer):
    "What is your name?"

@begin.subcommand
def quest(answer):
    "What is your quest?"

@begin.subcommand
def colour(answer):
    "What is your favourite colour?"

@begin.start
def main():
    pass



import begin
import logging

@begin.subcommand
def hello(name):
    "Say hello"
    logging.info("Hello {0}".format(name))

@begin.subcommand
def goodbye(name):
    "Say goodbye"
    logging.info("Goodbye {0}".format(name))

@begin.start
@begin.logging
def main():
    "Greetings and salutations"



"Command line extension plugins"
import cgitb
import logging
import logging.handlers
import platform
import sys

from begin.wrappable import Wrapping
from begin.utils import tobool

__all__ = ['logger', 'tracebacks']


class Extension(Wrapping):
    """Base class of all command line extensions

    Extension are required to subclass this class and override the
    add_arguments() and run() methods.
    """

    def add_arguments(self, parser, defaults):
        "Add command line arguments to parser"
        raise NotImplementedError("Command line extension incomplete")

    def run(self, opts):
        "run extension using command line options"
        raise NotImplementedError("Command line extension incomplete")


class Tracebacks(Extension):
    "Manage command line extension for cgitb module"

    section = 'tracebacks'

    def add_arguments(self, parser, defaults):
        "Add command line arguments for configuring traceback module"
        group = parser.add_argument_group('tracebacks',
                'Extended traceback reports on failure')
        group.add_argument('--tracebacks', action='store_true',
                default=tobool(defaults.from_name(
                    'enable', section=self.section, default='false')),
                help='Enable extended traceback reports')
        group.add_argument('--tbdir',
                default=defaults.from_name(
                    'directory', section=self.section, default=None),
                help='Write tracebacks to directory')

    def run(self, opts):
        "Configure cgitb module if enabled on command line"
        if not opts.tracebacks:
            return
        cgitb.enable(logdir=opts.tbdir, format='txt')


def tracebacks(func):
    "Add command line extension for cgitb module"
    return Tracebacks(func)


class Logging(Extension):
    "Manage command line extension for the logging module"

    section = 'logging'

    def add_arguments(self, parser, defaults):
        "Add command line arguments for configuring the logging module"
        exclusive  = parser.add_mutually_exclusive_group()
        exclusive.add_argument('-v', '--verbose',
                default=False, action='store_true',
                help='Increse logging output')
        exclusive.add_argument('-q', '--quiet',
                default=False, action='store_true',
                help='Decrease logging output')
        group = parser.add_argument_group('logging',
                'Detailed control of logging output')
        group.add_argument('--loglvl',
                default=defaults.from_name(
                    'level', section=self.section, default=None),
                choices=['DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL'],
                help='Set explicit log level')
        group.add_argument('--logfile',
                default=defaults.from_name(
                    'file', section=self.section, default=None),
                help='Ouput log messages to file')
        group.add_argument('--logfmt',
                default=defaults.from_name(
                    'format', section=self.section, default=None),
                help='Log message format')

    def run(self, opts):
        "Configure logging module according to command line options"
        # log level
        level = logging.INFO
        if opts.loglvl is not None:
            level = logging.getLevelName(opts.loglvl)
        elif opts.verbose:
            level = logging.DEBUG
        elif opts.quiet:
            level = logging.WARNING
        # logger
        logger = logging.getLogger()
        for handler in logger.handlers:
            logger.removeHandler(handler)
        logger.setLevel(level)
        # handler
        if opts.logfile is None:
            handler = logging.StreamHandler(sys.stdout)
        elif platform.system() != 'Windows':
            handler = logging.handlers.WatchedFileHandler(opts.logfile)
        else:
            handler = logging.FileHandler(opts.logfile)
        logger.addHandler(handler)
        # formatter
        fmt = opts.logfmt
        if fmt is None:
            if opts.logfile is None:
                fmt = '%(message)s'
            else:
                fmt = '[%(asctime)s] [%(levelname)s] [%(pathname)s:%(lineno)s] %(message)s'
        formatter = logging.Formatter(fmt)
        handler.setFormatter(formatter)


def logger(func):
    "Add command line extension for logging module"
    return Logging(func)



"Decorator utilities for wrapping functions"

__all__ = ['Wrapping']


class Wrapping(object):
    """Base class for function decorating objects

    Wraps a function, copying the functions special variables.
    """

    ASSIGNMENTS = ('__annotations__', '__doc__', '__module__', '__name__')

    def __init__(self, func):
        self.__wrapped__ = func
        for name in self.ASSIGNMENTS:
            setattr(self, name, getattr(func, name, None))

    def __call__(self, *args, **kwargs):
        return self.__wrapped__(*args, **kwargs)



from __future__ import absolute_import, division, print_function
import mock
import sys

try:
    import unittest2 as unittest
except ImportError:
    import unittest

from begin import utils


class TestToList(unittest.TestCase):

    def test_empty_list(self):
        self.assertEqual(utils.tolist(''), [])

    def test_default_list(self):
        self.assertEqual(utils.tolist('a,b'), ['a', 'b'])

    def test_empty_strings(self):
        self.assertEqual(utils.tolist(',', empty_strings=True), ['', ''])

    def test_custom_separator(self):
        self.assertEqual(utils.tolist('a|b', sep='|'), ['a', 'b'])

    def test_function_generator(self):
        func = utils.tolist()
        self.assertEqual(utils.tolist('a,b'), ['a', 'b'])



from __future__ import absolute_import, division, print_function
import mock

try:
    import unittest2 as unittest
except ImportError:
    import unittest

import pkg_resources

import begin


class TestSubCommand(unittest.TestCase):

    def tearDown(self):
        for collector in begin.subcommands.COLLECTORS.values():
            collector.clear()

    def test_register_default(self):
        @begin.subcommand
        def one():
            pass
        @begin.subcommand
        def two():
            pass
        self.assertEqual(list(begin.subcommands.COLLECTORS[None].commands()), [one, two])

    def test_register_custom(self):
        collector = begin.subcommands.Collector()
        @begin.subcommand(collector=collector)
        def one():
            pass
        @begin.subcommand(collector=collector)
        def two():
            pass
        self.assertEqual(list(collector.commands()), [one, two])

    def test_double_register(self):
        @begin.subcommand
        def subfunction():
            pass
        with self.assertRaises(ValueError):
            @begin.subcommand
            def subfunction():
                pass

    def test_get_registered(self):
        @begin.subcommand
        def function():
            pass
        self.assertIs(begin.subcommands.COLLECTORS[None].get('function'), function)

    @mock.patch('pkg_resources.iter_entry_points')
    def test_entry_points(self, iep):
        def function():
            pass
        def entry_points(group, *args):
            yield function
        iep.side_effect = entry_points
        collector = begin.subcommands.Collector()
        collector.load_plugins('entry.point')
        self.assertEqual(list(collector.commands()), [function])

    def test_named_collector(self):
        @begin.subcommand(group='named.collector')
        def function():
            pass
        self.assertEqual(len(begin.subcommands.COLLECTORS[None]), 0)
        self.assertIn('named.collector', begin.subcommands.COLLECTORS)
        self.assertIs(function,
                begin.subcommands.COLLECTORS['named.collector']['function'])



"""Utility functions for begins"""
from distutils.util import strtobool as _tobool
from argparse import FileType as tofile

def tobool(value):
    """Convert a string representation of truth to True or False.

    True values are 'y', 'yes', 't', 'true', 'on', and '1'; false values
    are 'n', 'no', 'f', 'false', 'off', and '0'.  Raises ValueError if
    'value' is anything else.
    """
    if isinstance(value, bool):
        return value
    return bool(_tobool(value))

def tolist(value=None, sep=',', empty_strings=False):
    """Convert a string to a list.

    The input string is split on the separator character. The default separator
    is ','. An alternative separator may be passed as the 'sep' keyword. If no
    string value is provided a function is returned that splits a string on the
    provided separator, or default if none was provided. Any empty strings are
    removed from the resulting list. This behaviour can be changed by passing
    True as the 'empty_strings' keyword argument.
    """
    def tolist(value):
        result = value.split(sep)
        if not empty_strings:
            return [r for r in result if len(r) > 0]
        return result
    if value is None:
        return tolist
    return tolist(value)



from __future__ import absolute_import, division, print_function
import cgitb
import logging
import mock
import os
import sys

try:
    import unittest2 as unittest
except ImportError:
    import unittest

import pkg_resources

import begin


class TestStart(unittest.TestCase):

    def tearDown(self):
        for collector in begin.subcommands.COLLECTORS.values():
            collector.clear()

    def test_start_false(self):
        # begin.start() return false outside __main__
        self.assertFalse(begin.start())

    def test_start_true(self):
        # mock globals to mimic __main__
        try:
            original = globals()['__name__']
            globals()['__name__'] = "__main__"
            self.assertTrue(begin.start())
        finally:
            globals()['__name__'] = original

    def test_decorate_false(self):
        target = mock.Mock()
        @begin.start
        def main():
            target()
        self.assertFalse(target.called)

    def test_decorate_true(self):
        target = mock.Mock()
        try:
            orig_argv= sys.argv
            sys.argv = orig_argv[:1]
            orig_name = globals()['__name__']
            globals()['__name__'] = "__main__"
            @begin.start
            def main():
                target()
            self.assertTrue(target.called)
            self.assertEqual(target.call_args[0], tuple())
        finally:
            sys.argv = orig_argv
            globals()['__name__'] = orig_name

    def test_decorate_callable(self):
        target = mock.Mock()
        @begin.start
        def main():
            target()
        main()
        self.assertTrue(target.called)

    def test_entry_point(self):
        target = mock.Mock()
        try:
            orig_argv= sys.argv
            sys.argv = orig_argv[:1]
            @begin.start
            def main():
                target()
            main.start()
            self.assertTrue(target.called)
        finally:
            sys.argv = orig_argv

    def test_command_line(self):
        target = mock.Mock()
        try:
            orig_argv= sys.argv
            sys.argv = orig_argv[:1] + ['-a', 'A']
            orig_name = globals()['__name__']
            globals()['__name__'] = "__main__"
            @begin.start
            def main(a, b=Ellipsis):
                target(a, b)
            self.assertTrue(target.called)
            self.assertEqual(target.call_args[0], ('A', Ellipsis))
        finally:
            sys.argv = orig_argv
            globals()['__name__'] = orig_name

    def test_env_disabled(self):
        target = mock.Mock()
        try:
            orig_env = os.environ
            os.environ = {'ALPHA_': 'A'}
            orig_argv= sys.argv
            sys.argv = orig_argv[:1]
            orig_name = globals()['__name__']
            globals()['__name__'] = "__main__"
            @begin.start
            def main(alpha_='Z', b=Ellipsis):
                target(alpha_, b)
            self.assertTrue(target.called)
            self.assertEqual(target.call_args[0], ('Z', Ellipsis))
        finally:
            os.environ = orig_env
            sys.argv = orig_argv
            globals()['__name__'] = orig_name

    def test_env_prefixes(self):
        target = mock.Mock()
        try:
            orig_env = os.environ
            os.environ = {'X_ALPHA_': 'A'}
            orig_argv= sys.argv
            sys.argv = orig_argv[:1]
            orig_name = globals()['__name__']
            globals()['__name__'] = "__main__"
            @begin.start(env_prefix='X_')
            def main(alpha_='Z', b=Ellipsis):
                target(alpha_, b)
            self.assertTrue(target.called)
            self.assertEqual(target.call_args[0], ('A', Ellipsis))
        finally:
            os.environ = orig_env
            sys.argv = orig_argv
            globals()['__name__'] = orig_name

    def test_not_callable(self):
        target = mock.Mock
        with self.assertRaises(ValueError):
            begin.start(Ellipsis)

    def test_convert(self):
        target = mock.Mock()
        try:
            orig_argv= sys.argv
            sys.argv = orig_argv[:1] + ['-a', '1']
            orig_name = globals()['__name__']
            globals()['__name__'] = "__main__"
            @begin.start
            @begin.convert(a=int)
            def main(a=0):
                target(a)
            self.assertTrue(target.called)
            self.assertEqual(target.call_args[0], (1,))
        finally:
            sys.argv = orig_argv
            globals()['__name__'] = orig_name

    @mock.patch('cgitb.enable')
    def test_tracebacks(self, enable):
        target = mock.Mock()
        try:
            orig_argv= sys.argv
            sys.argv = orig_argv[:1] + ['--tracebacks']
            orig_name = globals()['__name__']
            globals()['__name__'] = "__main__"
            @begin.start
            @begin.tracebacks
            def main():
                pass
            self.assertTrue(enable.called)
        finally:
            sys.argv = orig_argv
            globals()['__name__'] = orig_name

    @mock.patch('logging.getLogger')
    def test_logging(self, getlogger):
        target = mock.Mock()
        try:
            orig_argv= sys.argv
            sys.argv = orig_argv[:1]
            orig_name = globals()['__name__']
            globals()['__name__'] = "__main__"
            @begin.start
            @begin.logging
            def main():
                pass
            self.assertTrue(getlogger.called)
        finally:
            sys.argv = orig_argv
            globals()['__name__'] = orig_name

    def test_subcommand(self):
        epilogue = mock.Mock()
        target = mock.Mock()
        try:
            orig_argv= sys.argv
            sys.argv = orig_argv[:1] + ['subcmd']
            orig_name = globals()['__name__']
            globals()['__name__'] = "__main__"
            @begin.subcommand
            def subcmd():
                target()
            @begin.start
            def main():
                epilogue()
            self.assertTrue(epilogue.called)
            self.assertTrue(target.called)
        finally:
            sys.argv = orig_argv
            globals()['__name__'] = orig_name

    def test_subcommand_group(self):
        epilogue = mock.Mock()
        target = mock.Mock()
        try:
            orig_argv= sys.argv
            sys.argv = orig_argv[:1] + ['subcmd']
            orig_name = globals()['__name__']
            globals()['__name__'] = "__main__"
            @begin.subcommand(group='named.collector')
            def subcmd():
                target()
            @begin.start(sub_group='named.collector')
            def main():
                epilogue()
            self.assertTrue(epilogue.called)
            self.assertTrue(target.called)
        finally:
            sys.argv = orig_argv
            globals()['__name__'] = orig_name

    def test_multiple_subcommand(self):
        target = mock.Mock()
        try:
            orig_argv= sys.argv
            sys.argv = orig_argv[:1] + ['subcmd', '--', 'subcmd', '--', 'subcmd']
            orig_name = globals()['__name__']
            globals()['__name__'] = "__main__"
            @begin.subcommand
            def subcmd():
                target()
            @begin.start(cmd_delim='--')
            def main():
                pass
            self.assertEqual(target.call_count, 3)
        finally:
            sys.argv = orig_argv
            globals()['__name__'] = orig_name

    @mock.patch('pkg_resources.iter_entry_points')
    def test_plugins(self, iep):
        epilogue = mock.Mock()
        target = mock.Mock()
        try:
            orig_argv= sys.argv
            sys.argv = orig_argv[:1] + ['subcmd']
            orig_name = globals()['__name__']
            globals()['__name__'] = "__main__"
            def subcmd():
                target()
            def entry_points(group, *args):
                yield subcmd
            iep.side_effect = entry_points
            @begin.start(plugins='entry.points')
            def main():
                epilogue()
            self.assertTrue(epilogue.called)
            self.assertTrue(target.called)
        finally:
            sys.argv = orig_argv
            globals()['__name__'] = orig_name

    @mock.patch('sys.exit')
    def test_keyboard_interrupt_main(self, exit):
        try:
            orig_argv= sys.argv
            sys.argv = orig_argv[:1]
            orig_name = globals()['__name__']
            globals()['__name__'] = "__main__"
            @begin.start
            def main():
                raise KeyboardInterrupt
            self.assertTrue(exit.called)
            self.assertGreater(exit.call_args[0][0], 0)
        finally:
            sys.argv = orig_argv
            globals()['__name__'] = orig_name

    @mock.patch('sys.exit')
    def test_keyboard_interrupt_subcommand(self, exit):
        try:
            orig_argv= sys.argv
            sys.argv = orig_argv[:1] + ['subcmd']
            orig_name = globals()['__name__']
            globals()['__name__'] = "__main__"
            @begin.subcommand
            def subcmd():
                raise KeyboardInterrupt
            @begin.start
            def main():
                pass
            self.assertTrue(exit.called)
            self.assertGreater(exit.call_args[0][0], 0)
        finally:
            sys.argv = orig_argv
            globals()['__name__'] = orig_name

    def test_context(self):
        try:
            orig_argv= sys.argv
            sys.argv = orig_argv[:1] + ['one', '--', 'two', '--', 'three']
            orig_name = globals()['__name__']
            globals()['__name__'] = "__main__"
            @begin.subcommand
            def one():
                self.assertIs(begin.context.return_value, None)
                self.assertEqual(len(begin.context.opts_previous), 0)
                self.assertEqual(len(begin.context.opts_next), 2)
                return 1
            @begin.subcommand
            def two():
                self.assertEqual(begin.context.return_value, 1)
                self.assertEqual(len(begin.context.opts_previous), 1)
                self.assertEqual(len(begin.context.opts_next), 1)
                return 2
            @begin.subcommand
            def three():
                self.assertEqual(begin.context.return_value, 2)
                self.assertEqual(len(begin.context.opts_previous), 2)
                self.assertEqual(len(begin.context.opts_next), 0)
                return 3
            @begin.start(cmd_delim='--')
            def main():
                pass
            self.assertEqual(begin.context.return_value, 3)
            self.assertEqual(len(begin.context.opts_previous), 3)
            self.assertEqual(len(begin.context.opts_next), 0)
        finally:
            sys.argv = orig_argv
            globals()['__name__'] = orig_name

    def test_automatic_convert(self):
        target = mock.Mock()
        try:
            orig_argv= sys.argv
            sys.argv = orig_argv[:1] + ['-a', '-b', '1', '-c', 'x,y', '-d', '*']
            orig_name = globals()['__name__']
            globals()['__name__'] = "__main__"
            @begin.start(auto_convert=True)
            def main(a=False, b=0, c=(), d=''):
                target(a, b, c, d)
            self.assertTrue(target.called)
            self.assertEqual(target.call_args[0], (True, 1, ['x', 'y'], '*'))
            self.assertIsInstance(target.call_args[0][0], bool)
            self.assertIsInstance(target.call_args[0][1], int)
            self.assertIsInstance(target.call_args[0][2], list)
        finally:
            sys.argv = orig_argv
            globals()['__name__'] = orig_name


if __name__ == '__main__':
    unittest.begin()



from __future__ import absolute_import, division, print_function
import argparse
import cgitb
import logging
import logging.handlers
import mock
import platform
import sys

try:
    import unittest2 as unittest
except ImportError:
    import unittest

from begin import cmdline
from begin import extensions


def target():
    "target"
    return Ellipsis


class Options(object):
    pass


class TestExtension(unittest.TestCase):

    def setUp(self):
        self.extension = extensions.Extension(target)

    def test_call(self):
        self.assertIs(self.extension(), Ellipsis)

    def test_add_arguments(self):
        self.assertRaises(NotImplementedError, self.extension.add_arguments,
                mock.Mock(), mock.Mock())

    def test_run(self):
        self.assertRaises(NotImplementedError, self.extension.run, mock.Mock())

    def test_property_wrapped(self):
        self.assertIs(self.extension.__wrapped__, target)

    def test_property_doc(self):
        self.assertIs(self.extension.__doc__, target.__doc__)

    def test_property_module(self):
        self.assertIs(self.extension.__module__, target.__module__)

    def test_property_name(self):
        self.assertIs(self.extension.__name__, target.__name__)

    if sys.version_info[0] > 2:
        exec("""
def test_property_annotations(self):
    self.assertIs(self.extension.__annotations__, target.__annotations__)
""")


class TestTracebacks(unittest.TestCase):

    def setUp(self):
        self.tracebacks = extensions.Tracebacks(target)

    def test_add_arguments(self):
        parser = argparse.ArgumentParser()
        self.assertEqual(len(parser._action_groups), 2)
        self.assertEqual(len(parser._optionals._actions), 1)
        self.tracebacks.add_arguments(parser, cmdline.DefaultsManager())
        self.assertEqual(len(parser._action_groups), 3)
        self.assertEqual(len(parser._optionals._actions), 3)

    @mock.patch('cgitb.enable')
    def test_run_disabled(self, enable):
        opts = Options()
        opts.tracebacks = False
        self.tracebacks.run(opts)
        self.assertFalse(enable.called)

    @mock.patch('cgitb.enable')
    def test_run_stdout(self, enable):
        opts = Options()
        opts.tracebacks = True
        opts.tbdir = None
        self.tracebacks.run(opts)
        enable.assert_called_once_with(format='txt', logdir=None)

    @mock.patch('cgitb.enable')
    def test_run_directory(self, enable):
        opts = Options()
        opts.tracebacks = True
        opts.tbdir = '/dev/null'
        self.tracebacks.run(opts)
        enable.assert_called_once_with(format='txt', logdir='/dev/null')


class TestLoging(unittest.TestCase):

    def setUp(self):
        self.logs = extensions.Logging(target)
        self.opts = Options()
        self.opts.verbose = False
        self.opts.quiet = False
        self.opts.loglvl = None
        self.opts.logfile = None
        self.opts.logfmt = None

    def test_add_arguments(self):
        parser = argparse.ArgumentParser()
        self.assertEqual(len(parser._action_groups), 2)
        self.assertEqual(len(parser._optionals._actions), 1)
        self.logs.add_arguments(parser, cmdline.DefaultsManager())
        self.assertEqual(len(parser._action_groups), 3)
        self.assertEqual(len(parser._optionals._actions), 6)

    @mock.patch('logging.getLogger')
    def test_run_default(self, getlogger):
        self.logs.run(self.opts)
        self.assertTrue(getlogger.called)
        logger = getlogger.return_value
        logger.setLevel.assert_called_once_with(logging.getLevelName('INFO'))
        self.assertTrue(logger.addHandler.called)
        self.assertIsInstance(logger.addHandler.call_args[0][0],
                logging.StreamHandler)

    @mock.patch('logging.getLogger')
    def test_run_verbose(self, getlogger):
        self.opts.verbose = True
        self.logs.run(self.opts)
        self.assertTrue(getlogger.called)
        logger = getlogger.return_value
        logger.setLevel.assert_called_once_with(logging.getLevelName('DEBUG'))

    @mock.patch('logging.getLogger')
    def test_run_quiet(self, getlogger):
        self.opts.quiet = True
        self.logs.run(self.opts)
        self.assertTrue(getlogger.called)
        logger = getlogger.return_value
        logger.setLevel.assert_called_once_with(logging.getLevelName('WARNING'))

    @mock.patch('logging.getLogger')
    def test_run_loglvl(self, getlogger):
        self.opts.loglvl = 'ERROR'
        self.logs.run(self.opts)
        self.assertTrue(getlogger.called)
        logger = getlogger.return_value
        logger.setLevel.assert_called_once_with(logging.getLevelName('ERROR'))

    @mock.patch('logging.getLogger')
    def test_run_reinitialise(self, getlogger):
        getlogger.return_value.handlers = [Ellipsis]
        self.logs.run(self.opts)
        getlogger.return_value.removeHandler.assert_called_once_with(Ellipsis)

    @mock.patch('logging.open', mock.mock_open(), create=True)
    @mock.patch('platform.system')
    @mock.patch('logging.getLogger')
    def test_run_logfile_linux(self, getlogger, system):
        system.return_value = 'Linux'
        self.opts.logfile = '/dev/null'
        self.logs.run(self.opts)
        logger = getlogger.return_value
        self.assertTrue(logger.addHandler.called)
        self.assertIsInstance(logger.addHandler.call_args[0][0],
                logging.handlers.WatchedFileHandler)

    @mock.patch('logging.open', mock.mock_open(), create=True)
    @mock.patch('platform.system')
    @mock.patch('logging.getLogger')
    def test_run_logfile_windows(self, getlogger, system):
        system.return_value = 'Windows'
        self.opts.logfile = '/dev/null'
        self.logs.run(self.opts)
        logger = getlogger.return_value
        self.assertTrue(logger.addHandler.called)
        self.assertIsInstance(logger.addHandler.call_args[0][0],
                logging.FileHandler)


if __name__ == '__main__':
    unittest.begin()



from __future__ import absolute_import, division, print_function
import cgitb
import mock
import os
import sys

try:
    import unittest2 as unittest
except ImportError:
    import unittest

import pkg_resources

from begin import cmdline
from begin import extensions
from begin import subcommands


class Options(object):
    pass


class TestProgramName(unittest.TestCase):

    def setUp(self):
        self.ordir = os.path.abspath(os.curdir)
        os.chdir('./tests')

    def tearDown(self):
        os.chdir(self.ordir)

    def test_simple_script(self):
        def func():
            pass
        prog = cmdline.program_name('main.py', func)
        self.assertEqual(prog, 'main.py')

    def test_main_module(self):
        def func():
            pass
        prog = cmdline.program_name('__main__.py', func)
        self.assertEqual(prog, 'tests')

    @mock.patch('os.path.split')
    def test_unusable_path(self, psplit):
        psplit.return_value = ('', '__main__.py')
        def func():
            pass
        prog = cmdline.program_name('__main__.py', func)
        self.assertEqual(prog, 'func')


class TestCreateParser(unittest.TestCase):

    def tearDown(self):
        for collector in subcommands.COLLECTORS.values():
            collector.clear()

    def test_void_function(self):
        def main():
            pass
        parser = cmdline.create_parser(main)
        self.assertEqual(len(parser._optionals._actions), 1)
        self.assertEqual(parser._optionals._actions[0].dest, 'help')

    def test_positional_arguments(self):
        def main(a, b, c):
            pass
        parser = cmdline.create_parser(main)
        self.assertEqual(len(parser._optionals._actions), 4)
        self.assertIs(parser._optionals._actions[1].default, cmdline.NODEFAULT)

    def test_keyword_argument(self):
        def main(opt=Ellipsis):
            pass
        parser = cmdline.create_parser(main)
        self.assertEqual(len(parser._optionals._actions), 2)
        self.assertIs(parser._optionals._actions[1].default, Ellipsis)

    def test_variable_arguments(self):
        def main(*variable_arguments):
            pass
        parser = cmdline.create_parser(main)
        self.assertIn('variable_arguments', parser.format_help())

    def test_function_description(self):
        def main(a):
            'program description'
        parser = cmdline.create_parser(main)
        self.assertEqual(parser.description, main.__doc__)

    if sys.version_info[0] > 2:
        exec("""
def test_positional_with_annotation(self):
    def main(opt:'help string'):
        pass
    parser = cmdline.create_parser(main)
    self.assertEqual(len(parser._optionals._actions), 2)
    self.assertEqual(parser._optionals._actions[1].help, 'help string')

def test_positional_with_annotation_and_default(self):
    def main(opt:'help string'='default'):
        pass
    parser = cmdline.create_parser(main)
    self.assertEqual(len(parser._optionals._actions), 2)
    self.assertEqual(parser._optionals._actions[1].help,
        'help string (default: %(default)s)')

def test_variable_positional_with_annotation(self):
    def main(*opts:'help string'):
        pass
    parser = cmdline.create_parser(main)
    self.assertEqual(len(parser._optionals._actions), 2)
    self.assertEqual(parser._optionals._actions[1].help, 'help string')
""")

    def test_variable_keyword_arguments(self):
        def main(**args):
            pass
        with self.assertRaises(ValueError):
            cmdline.create_parser(main)

    def test_env_disabled(self):
        def main(one, two):
            pass
        try:
            original = os.environ
            os.environ = {'ONE': 1}
            parser = cmdline.create_parser(main)
            self.assertEqual(parser._optionals._actions[1].default, cmdline.NODEFAULT)
            self.assertIs(parser._optionals._actions[2].default, cmdline.NODEFAULT)
        finally:
            os.environ = original

    def test_env_prefixes(self):
        def main(one, two):
            pass
        try:
            original = os.environ
            os.environ = {'X_ONE': 1, 'TWO': 2}
            parser = cmdline.create_parser(main, env_prefix='X_')
            self.assertEqual(parser._optionals._actions[1].default, 1)
            self.assertIs(parser._optionals._actions[2].default, cmdline.NODEFAULT)
        finally:
            os.environ = original

    @mock.patch('os.path.expanduser')
    def test_configfile(self, expanduser):
        def main(arg, unk):
            pass
        expanduser.return_value = os.path.join(os.curdir, 'tests')
        parser = cmdline.create_parser(main, config_file='config_test.cfg')
        self.assertEqual(parser._optionals._actions[1].default, 'value')
        self.assertEqual(parser._optionals._actions[2].default, cmdline.NODEFAULT)

    def test_help_strings(self):
        def main(a, b=None):
            pass
        parser = cmdline.create_parser(main)
        self.assertIs(parser._optionals._actions[1].help, None)
        self.assertEqual(parser._optionals._actions[2].help, '(default: %(default)s)')

    def test_tracebacks(self):
        @extensions.Tracebacks
        def main():
            pass
        parser = cmdline.create_parser(main)
        self.assertEqual(len(parser._action_groups), 3)
        self.assertEqual(len(parser._optionals._actions), 3)

    def test_subcommand(self):
        @subcommands.subcommand
        def subcmd():
            pass
        def main():
            pass
        parser = cmdline.create_parser(main)
        self.assertEqual(len(parser._action_groups), 3)
        self.assertIn('subcmd', parser.format_help())

    def test_subcommand_group(self):
        @subcommands.subcommand(group='named.collector')
        def subcmd():
            pass
        def main():
            pass
        parser = cmdline.create_parser(main, sub_group='named.collector')
        self.assertEqual(len(parser._action_groups), 3)
        self.assertIn('subcmd', parser.format_help())

    @mock.patch('pkg_resources.iter_entry_points')
    def test_plugins(self, iep):
        def main():
            pass
        def subcmd():
            pass
        def entry_points(group, *args):
            yield subcmd
        iep.side_effect = entry_points
        parser = cmdline.create_parser(main, plugins='entry.point')
        self.assertEqual(len(parser._action_groups), 3)
        self.assertIn('subcmd', parser.format_help())

    def test_conflict_resoltion(self):
        def main(help):
            pass
        parser = cmdline.create_parser(main)

    def test_subcommand_conflict_resoltion(self):
        @subcommands.subcommand
        def subcmd(help):
            pass
        def main():
            pass
        parser = cmdline.create_parser(main)

    def test_no_short_args(self):
        def main(alpha):
            pass
        parser = cmdline.create_parser(main, short_args=False)
        self.assertNotIn(' -a ', parser.format_help())

    def test_lexical_order(self):
        def main(zeta, alpha, delta):
            pass
        parser = cmdline.create_parser(main, lexical_order=True)
        self.assertEqual(parser._optionals._actions[1].dest, 'alpha')
        self.assertEqual(parser._optionals._actions[2].dest, 'delta')
        self.assertEqual(parser._optionals._actions[3].dest, 'zeta')

    def test_subcommand_description(self):
        @subcommands.subcommand
        def subcmd():
            """SUBCOMMAND_HELP

            SUBCOMMAND_DESC
            """
        def main():
            pass
        parser = cmdline.create_parser(main)
        self.assertIn('SUBCOMMAND_HELP', parser.format_help())
        self.assertNotIn('SUBCOMMAND_DESC', parser.format_help())


class TestApplyOptions(unittest.TestCase):

    def setUp(self):
        self.opts = Options()

    def tearDown(self):
        for collector in subcommands.COLLECTORS.values():
            collector.clear()

    def test_void_function(self):
        def main():
            return Ellipsis
        value = cmdline.apply_options(main, self.opts)
        self.assertIs(value, Ellipsis)

    def test_positional_argument(self):
        def main(a, b,c):
            return (a, b, c)
        self.opts.a = 1
        self.opts.b = 2
        self.opts.c = 3
        value = cmdline.apply_options(main, self.opts)
        self.assertTupleEqual(value, (1, 2, 3))

    def test_keyword_argument(self):
        def main(a=None, b=None, c=None):
            return (a, b, c)
        self.opts.a = 1
        self.opts.b = 2
        self.opts.c = 3
        value = cmdline.apply_options(main, self.opts)
        self.assertEqual(value, (1, 2, 3))

    def test_variable_arguments(self):
        def main(*args):
            return tuple(args)
        self.opts.args = [1, 2, 3]
        value = cmdline.apply_options(main, self.opts)
        self.assertTupleEqual(value, (1, 2, 3))

    def test_variable_keywords(self):
        def main(**kwargs):
            return dict(args)
        with self.assertRaises(cmdline.CommandLineError):
            value = cmdline.apply_options(main, self.opts)

    def test_positional_with_variable_arguments(self):
        def main(a, *b):
            return (a, b)
        self.opts.a = 1
        self.opts.b = [2]
        value = cmdline.apply_options(main, self.opts)
        self.assertTupleEqual(value, (1, (2,)))

    def test_missing_option(self):
        def main(a):
            return a
        with self.assertRaises(cmdline.CommandLineError):
            value = cmdline.apply_options(main, self.opts)

    @mock.patch('sys.stderr')
    @mock.patch('sys.exit')
    def test_missing_required(self, exit, stderr):
        def main(a):
            pass
        self.opts.a = cmdline.NODEFAULT
        cmdline.apply_options(main, self.opts)
        self.assertTrue(exit.called)

    if sys.version_info[0] > 2:
        exec("""
def test_keyword_only(self):
    def main(*, a=None):
        return a
    self.opts.a = 1
    value = cmdline.apply_options(main, self.opts)
    self.assertEqual(value, 1)
""")

    @mock.patch('cgitb.enable')
    def test_tracebacks(self, enable):
        @extensions.tracebacks
        def main():
            return
        self.opts.tracebacks = True
        self.opts.tbdir = None
        cmdline.apply_options(main, self.opts)
        enable.assert_called_one(format='txt', logdir=None)

    def test_subcommand(self):
        @subcommands.subcommand
        def subcmd():
            return 'blue'
        def main():
            return 'red'
        self.opts._subcommand = 'subcmd'
        value = cmdline.apply_options(main, self.opts)
        self.assertEqual('blue', value)

    def test_subcommand_group(self):
        @subcommands.subcommand(group='named.collector')
        def subcmd():
            return 'blue'
        def main():
            return 'red'
        self.opts._subcommand = 'subcmd'
        value = cmdline.apply_options(main, self.opts, sub_group='named.collector')
        self.assertEqual('blue', value)


if __name__ == "__main__":
    unittest.begin()



"Generate command line parsers and apply options using function signatures"
import argparse
import os
import sys

try:
    import configparser
except ImportError:
    import ConfigParser as configparser

try:
    from inspect import signature
except ImportError:
    from funcsigs import signature

from begin import context, extensions, subcommands, utils

__all__ = ['create_parser', 'populate_parser',
           'apply_options', 'call_function']


NODEFAULT = object()


class CommandLineError(ValueError):
    """Error in command line processing"""


class DefaultsManager(object):
    """Manage default values for command line options

    Inspects environment variables and default argument values to determine the
    correct default for command line option.
    """

    def __init__(self, env_prefix=None, config_file=None, config_section=None):
        self._use_env = env_prefix is not None
        self._prefix = '' if not self._use_env else env_prefix
        self._parser = configparser.ConfigParser()
        self._section = config_section
        if config_file is not None:
            self._parser.read([config_file,
                os.path.join(os.path.expanduser('~'), config_file)])

    def metavar(self, name):
        "Generate meta variable name for parameter"
        metavar = (self._prefix + name).upper()
        return metavar

    def from_param(self, param, default=NODEFAULT):
        "Get default value from signature paramater"
        if param.default is not param.empty:
            default = param.default
        default = self.from_name(param.name, default)
        return default

    def from_name(self, name, default=NODEFAULT, section=None):
        "Get default value from argument name"
        if len(self._parser.sections()) > 0:
            section = self._section if section is None else section
            try:
                default = self._parser.get(section, name)
            except (configparser.NoSectionError, configparser.NoOptionError):
                pass
        if self._use_env:
            default = os.environ.get(self.metavar(name), default)
        return default

    def set_config_section(self, section):
        self._section = section


def program_name(filename, func):
    """Choose program name for application

    Iterate backwards through absolute path for application looking for first
    name that is not a magic variable.
    """
    fullpath = os.path.abspath(filename)
    basename, filename = os.path.split(fullpath)
    while len(basename) > 0:
        if not filename.startswith('__'):
            return filename
        basename, filename = os.path.split(basename)
    return func.__name__


def populate_parser(parser, defaults, funcsig, short_args, lexical_order):
    """Populate parser according to function signature

    Use the parameters accepted by the source function, according to the
    functions signature provided, to populating a corresponding command line
    argument parser.
    """
    params = funcsig.parameters.values()
    if lexical_order:
        params = sorted(params, key=lambda p: p.name)
    for param in params:
        if param.kind == param.POSITIONAL_OR_KEYWORD or \
                param.kind == param.KEYWORD_ONLY or \
                param.kind == param.POSITIONAL_ONLY:
            kwargs = {'default': defaults.from_param(param)}
            if isinstance(param.default, bool):
                if not isinstance(kwargs['default'], bool):
                    kwargs['default'] = utils.tobool(kwargs['default'])
                if kwargs['default']:
                    kwargs['action'] = 'store_false'
                else:
                    kwargs['action'] = 'store_true'
            else:
                kwargs['metavar'] = defaults.metavar(param.name)
            if param.annotation is not param.empty:
                kwargs['help'] = param.annotation
            if kwargs['default'] is NODEFAULT:
                kwargs['required'] = True
            else:
                if 'help' not in kwargs:
                    kwargs['help'] = '(default: %(default)s)'
                else:
                    kwargs['help'] += ' (default: %(default)s)'
            args = ['--' + param.name]
            if short_args:
                args.append('-' + param.name[0])
            parser.add_argument(*args, **kwargs)
        elif param.kind == param.VAR_POSITIONAL:
            kwargs = {'nargs': '*'}
            if param.annotation is not param.empty:
                kwargs['help'] = param.annotation
            parser.add_argument(param.name, **kwargs)
        elif param.kind == param.VAR_KEYWORD:
            msg = 'Variable length keyword arguments not supported'
            raise ValueError(msg)
    return parser


def create_parser(func, env_prefix=None, config_file=None, config_section=None,
        short_args=True, lexical_order=False, sub_group=None, plugins=None,
        collector=None):
    """Create and OptionParser object from a function definition.

    Use the function's signature to generate an OptionParser object. Default
    values are honoured, argument annotations are used as help strings and the
    functions docstring becomes the parser description. Environment variables
    can alter the default values of options. Variable positional arguments are
    ingored but will alter the program's usage string. Variable keyword
    arguments will raise a ValueError exception. A prefix on expected
    environment variables can be added using the env_prefix argument.
    """
    defaults = DefaultsManager(env_prefix, config_file, func.__name__)
    parser = argparse.ArgumentParser(
            prog=program_name(sys.argv[0], func),
            argument_default=NODEFAULT,
            conflict_handler='resolve',
            description = func.__doc__
    )
    collector = collector if collector is not None else subcommands.COLLECTORS[sub_group]
    if plugins is not None:
        collector.load_plugins(plugins)
    if len(collector) > 0:
        subparsers = parser.add_subparsers(title='Available subcommands',
                dest='_subcommand')
        for subfunc in collector.commands():
            funcsig  = signature(subfunc)
            help = None
            if subfunc.__doc__ is not None:
                help = subfunc.__doc__.splitlines()[0]
            subparser = subparsers.add_parser(subfunc.__name__, help=help,
                    conflict_handler='resolve', description=subfunc.__doc__)
            defaults.set_config_section(subfunc.__name__)
            populate_parser(subparser, defaults, funcsig, short_args, lexical_order)
    have_extensions = False
    while hasattr(func, '__wrapped__') and not hasattr(func, '__signature__'):
        if isinstance(func, extensions.Extension):
            func.add_arguments(parser, defaults)
            have_extensions = True
        func = getattr(func, '__wrapped__')
    funcsig = signature(func)
    populate_parser(parser, defaults, funcsig, short_args, lexical_order)
    return parser


def call_function(func, funcsig, opts):
    """Call function using command line options and arguments

    Use the function's signature to extract expected values from the provided
    command line options. The extracted values are used to call the funciton.
    The presence of variable keyword arguments, missing attributes from the
    options object or to failure to use the command line arguments list will
    result in a CommandLineError being raised.
    """
    def getoption(opts, name, default=None):
        if not hasattr(opts, name):
            msg = "Missing command line options '{0}'".format(name)
            raise CommandLineError(msg)
        value = getattr(opts, name)
        if value is NODEFAULT:
            if default is None:
                msg = "'{0}' is a required option{1}".format(name, os.linesep)
                sys.stderr.write(msg)
                sys.exit(1)
            else:
                value = default
        return value
    pargs = []
    kwargs = {}
    for param in funcsig.parameters.values():
        if param.kind == param.POSITIONAL_OR_KEYWORD or \
                param.kind == param.POSITIONAL_ONLY:
            pargs.append(getoption(opts, param.name))
        elif param.kind == param.VAR_POSITIONAL:
            pargs.extend(getoption(opts, param.name, []))
        elif param.kind == param.KEYWORD_ONLY:
            kwargs[param.name] = getoption(opts, param.name)
        elif param.kind == param.VAR_KEYWORD:
            msg = 'Variable length keyword arguments not supported'
            raise CommandLineError(msg)
    return func(*pargs, **kwargs)


def apply_options(func, opts, sub_group=None, collector=None):
    """Apply command line options to function and subcommands

    Call the target function, and any chosen subcommands, using the parsed
    command line arguments.
    """
    ext = func
    collector = collector if collector is not None else subcommands.COLLECTORS[sub_group]
    while hasattr(ext, '__wrapped__') and not hasattr(ext, '__signature__'):
        if isinstance(ext, extensions.Extension):
            ext.run(opts)
        ext = getattr(ext, '__wrapped__')
    return_value = call_function(func, signature(ext), opts)
    if return_value is not None:
        context.return_value = return_value
    if hasattr(opts, '_subcommand'):
        subfunc = collector.get(opts._subcommand)
        return_value = call_function(subfunc, signature(subfunc), opts)
        if return_value is not None:
            context.return_value = return_value
    return context.return_value



from __future__ import absolute_import, division, print_function
import mock
import sys

try:
    from collections.abc import Sequence
except ImportError:
    from collections import Sequence

try:
    import unittest2 as unittest
except ImportError:
    import unittest

import begin


class TestConvert(unittest.TestCase):

    def test_positional_as_positional(self):
        target = mock.Mock()
        @begin.convert(b=float)
        def func(a, b, c):
            target(a, b, c)
        func(None, '1.1', None)
        self.assertEqual(target.call_args[0], (None, 1.1, None))
        self.assertIsInstance(target.call_args[0][1], float)

    def test_positional_as_keyword(self):
        target = mock.Mock()
        @begin.convert(b=float)
        def func(a, b, c):
            target(a, b, c)
        func(None, b='1.1', c=None)
        self.assertEqual(target.call_args[0], (None, 1.1, None))
        self.assertIsInstance(target.call_args[0][1], float)

    def test_keyword_as_keyword(self):
        target = mock.Mock()
        @begin.convert(b=float)
        def func(a, b, c):
            target(a, b, c)
        func(None, b='1.1', c=None)
        self.assertEqual(target.call_args[0], (None, 1.1, None))
        self.assertIsInstance(target.call_args[0][1], float)

    def test_variable_positional(self):
        target = mock.Mock()
        @begin.convert(b=float)
        def func(a, *b):
            target(a, b)
        func(None, '1.1', '1.2')
        self.assertEqual(target.call_args[0], (None, (1.1, 1.2)))
        self.assertIsInstance(target.call_args[0][1][0], float)
        self.assertIsInstance(target.call_args[0][1][1], float)

    def test_variable_keyword(self):
        @begin.convert(b=float)
        def func(a, **b):
            target(a, b)
        with self.assertRaises(ValueError):
            func(None, b=None, c=None)

    def test_default_keyword(self):
        target = mock.Mock()
        @begin.convert(b=float)
        def func(a=None, b='x' * 32):
            target(a, b)
        func()
        self.assertEqual(target.call_args[0], (None, 'x' * 32))

    def test_defaults_as_arguments(self):
        target = mock.Mock()
        @begin.convert(port=int, debug=begin.utils.tobool)
        def func(host='0.0.0.0', port=80, debug=False):
            target(host, port, debug)
        func('0.0.0.0', '8080', False)
        self.assertEqual(target.call_args[0], ('0.0.0.0', 8080, False))

    if sys.version_info[0] > 2:
        exec("""
def test_keyword_only(self):
    target = mock.Mock()
    @begin.convert(b=float)
    def func(a, *, b=None):
        target(a, b)
    func(None, b='1.1')
    self.assertEqual(target.call_args[0], (None, 1.1))
    self.assertIsInstance(target.call_args[0][1], float)
""")

    def test_automatic_integer(self):
        target = mock.Mock()
        @begin.convert(_automatic=True)
        def func(number=0):
            target(number)
        func('1')
        self.assertIsInstance(target.call_args[0][0], int)
        self.assertEqual(target.call_args[0][0], 1)

    if sys.version_info[0] < 3:
        exec("""
def test_automatic_long(self):
    target = mock.Mock()
    @begin.convert(_automatic=True)
    def func(number=0L):
        target(number)
    func('1')
    self.assertIsInstance(target.call_args[0][0], long)
    self.assertEqual(target.call_args[0][0], 1)
""")

    def test_automatic_float(self):
        target = mock.Mock()
        @begin.convert(_automatic=True)
        def func(number=0.0):
            target(number)
        func('1')
        self.assertIsInstance(target.call_args[0][0], float)
        self.assertEqual(target.call_args[0][0], 1.0)

    def test_automatic_boolean(self):
        target = mock.Mock()
        @begin.convert(_automatic=True)
        def func(boolean=False):
            target(boolean)
        func('True')
        self.assertIsInstance(target.call_args[0][0], bool)
        self.assertEqual(target.call_args[0][0], True)

    def test_automatic_tuple(self):
        target = mock.Mock()
        @begin.convert(_automatic=True)
        def func(sequence=()):
            target(sequence)
        func('a,b')
        self.assertIsInstance(target.call_args[0][0], Sequence)
        self.assertSequenceEqual(target.call_args[0][0], ('a', 'b'))

    def test_automatic_list(self):
        target = mock.Mock()
        @begin.convert(_automatic=True)
        def func(sequence=[]):
            target(sequence)
        func('a,b')
        self.assertIsInstance(target.call_args[0][0], Sequence)
        self.assertSequenceEqual(target.call_args[0][0], ('a', 'b'))



#!/usr/bin/env python
from setuptools import setup
import re
import sys


def load_version(filename='begin/version.py'):
    "Parse a __version__ number from a source file"
    with open(filename) as source:
        text = source.read()
        match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", text)
        if not match:
            msg = "Unable to find version number in {}".format(filename)
            raise RuntimeError(msg)
        version = match.group(1)
        return version

PYTHON3K = sys.version_info[0] > 2

requires = ['funcsigs'] if sys.version_info[:2] < (3, 3) else []
requires += ['argparse'] if sys.version_info[:2] < (2, 7) else []

setup(
    name="begins",
    version=load_version(),
    packages=['begin'],
    zip_safe=False,
    author="Aaron Iles",
    author_email="aaron.iles@gmail.com",
    url="http://begins.readthedocs.org",
    description="Command line programs for busy developers",
    long_description=open('README.rst').read(),
    license="ASL",
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    install_requires=requires,
    tests_require=['mock'] + [] if PYTHON3K else ['unittest2'],
    test_suite="tests" if PYTHON3K else "unittest2.collector"
)



"Subcommand support for command lines"
from collections import defaultdict

try:
    from collections.abc import MutableMapping
except ImportError:
    from collections import MutableMapping

import pkg_resources


class Collector(dict):
    """Collect functions for use as subcommands"""

    def commands(self):
        for name in sorted(self):
            yield self[name]

    def load_plugins(self, entry_point):
        for func in pkg_resources.iter_entry_points(group=entry_point):
            self.register(func)

    def register(self, func):
        if func.__name__ in self:
            msg = "Function named '{0}' already registered".format(func.__name__)
            raise ValueError(msg)
        self[func.__name__] = func


def subcommand(func=None, group=None, collector=None):
    """Register function as a program's subcommand

    Functions are registered with the default subcommand collector, unless a
    custom collector is provided through the 'collector' argument. Subcommands
    will automatically be included in the generated command line support.
    """
    def wrapper(func):
        collector.register(func)
        return func
    collector = COLLECTORS[group] if collector is None else collector
    if func is not None:
        return wrapper(func)
    return wrapper


COLLECTORS = defaultdict(Collector)



import shutil
import begin

begin.subcommand(shutil.copy)
begin.subcommand(shutil.copy2)
begin.subcommand(shutil.copyfile)
begin.subcommand(shutil.copymode)
begin.subcommand(shutil.copystat)
begin.subcommand(shutil.copytree)
begin.subcommand(shutil.move)
begin.subcommand(shutil.rmtree)

# Patch in doc before the func is wrapped by begin.start
def patch_doc(doc):
   def decorate(func):
       func.__doc__ = doc
       return func
   return decorate

@begin.start
@patch_doc(shutil.__doc__)
def main():
   pass



"Type casting for function arguments"
from __future__ import absolute_import, division, print_function
import functools
import sys

try:
    from inspect import signature
except ImportError:
    from funcsigs import signature

from begin import utils

__all__ = ['convert']


CONVERTERS = {
        int: lambda _: int,
        float: lambda _: float,
        bool: lambda _: utils.tobool,
        tuple: lambda _: utils.tolist,
        list: lambda _: utils.tolist,
        type(sys.stdin): lambda src: functools.partial(utils.tofile, mode=src.mode),
}

if sys.version_info[0] < 3:
    CONVERTERS[long] = lambda _: long


def convert(_automatic=False, **mappings):
    """Cast function arguments to decorated function.

    Optionally use callables to cast the arguments for the decorated functions.

    >>> import begin
    >>> @begin.convert(second=int)
    ... def func(first, second=None):
    ...     pass

    If a value is passed as the 'second' argument to 'func', it will be
    replaced with the result of calling the 'int' function with the value. If
    no value is passed, the default value will be used without casting.

    Casting also works for variable position arguments. In this case the
    casting function is called on each argument.

    >>> @begin.convert(args=float)
    ... def func(*args):
    ...    pass

    Helper functions for casting arguments can be found in the 'begin.utils'
    module.
    """
    def decorator(func):
        target = func
        while hasattr(target, '__wrapped__'):
            target = getattr(func, '__wrapped__')
        sig = signature(target)
        if _automatic:
            for param in sig.parameters.values():
                if param.name in mappings or param.default is param.empty:
                    continue
                converter = CONVERTERS.get(type(param.default))
                if converter:
                    mappings[param.name] = converter(param.default)
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            args = list(args)
            for pos, param in enumerate(sig.parameters.values()):
                if param.name not in mappings:
                    continue
                if param.kind == param.POSITIONAL_ONLY:
                    args[pos] = mappings[param.name](args[pos])
                if param.kind == param.POSITIONAL_OR_KEYWORD:
                    if param.name in kwargs and \
                            kwargs[param.name] is not param.default:
                        kwargs[param.name] = mappings[param.name](
                                kwargs[param.name])
                    elif pos < len(args) and \
                            args[pos] is not param.default:
                        args[pos] = mappings[param.name](args[pos])
                if param.kind == param.KEYWORD_ONLY and \
                        kwargs[param.name] is not param.default:
                    kwargs[param.name] = mappings[param.name](
                            kwargs[param.name])
                if param.kind == param.VAR_POSITIONAL:
                    start = pos
                    for pos in range(start, len(args)):
                        args[pos] = mappings[param.name](args[pos])
                if param.kind == param.VAR_KEYWORD:
                    msg = 'Variable length keyword arguments not supported'
                    raise ValueError(msg)
            return func(*args, **kwargs)
        if not hasattr(wrapper, '__wrapped__'):
            wrapper.__wrapped__ = func
        return wrapper
    return decorator
